# -*- coding: utf-8 -*-
"""
Production Over Time Panel
==========================
Analyze top authors, sources, keywords, countries production over time.
"""

import tkinter as tk
from tkinter import ttk, messagebox, filedialog
import threading
import os
from typing import Dict, List, Optional

from biblium.gui.config import FONTS, LAYOUT, get_theme
from biblium.gui.core.events import event_bus
from biblium.gui.panels.base import BasePanel
from biblium.gui.widgets.cards import Card
from biblium.gui.widgets.buttons import ThemedButton, ActionButton
from biblium.gui.widgets.forms import LabeledCombobox, LabeledSpinbox, LabeledCheckbox
from biblium.gui.widgets.tables import DataTable

try:
    import pandas as pd
    import numpy as np
    HAS_PANDAS = True
except ImportError:
    HAS_PANDAS = False

try:
    import matplotlib.pyplot as plt
    HAS_MATPLOTLIB = True
except ImportError:
    HAS_MATPLOTLIB = False

# Available items for production over time analysis
# Format: (Display Name, key, [possible column names])
AVAILABLE_ITEMS_CONFIG = [
    ("Authors", "authors", ["Authors", "Author full names", "AU", "AF", "Author Full Names"]),
    ("Author Keywords", "author keywords", ["Author Keywords", "DE", "Keywords"]),
    ("Index Keywords", "index keywords", ["Index Keywords", "ID", "Keywords Plus", "Keyword Plus"]),
    ("Sources", "sources", ["Source title", "Source", "SO", "Journal", "Publication Name"]),
    ("Countries", "all countries", ["Countries", "Country", "Countries of Authors"]),
    ("Affiliations", "affiliations", ["Affiliations", "C1", "Addresses", "Author Affiliations"]),
    ("Document Types", "document types", ["Document Type", "DT", "Document type", "Publication Type", "Type"]),
    ("References", "references", ["References", "Cited References", "CR", "Referenced Works"]),
    ("Cited Sources", "cited sources", ["Cited Sources", "Cited Journals", "Cited Source"]),
    ("Cited Authors", "cited authors", ["Cited Authors", "Cited Author", "First Authors of Cited References"]),
    ("Research Areas", "research areas", ["Research Areas", "WoS Categories", "SC", "Subject Area", "Fields"]),
    ("Subject Categories", "subject categories", ["Web of Science Categories", "WC", "Subject Categories", "Categories"]),
]

def get_available_items_for_df(df):
    """Return list of available items based on columns in dataframe."""
    if df is None:
        return [(name, key) for name, key, _ in AVAILABLE_ITEMS_CONFIG]
    
    available = []
    columns_lower = {c.lower(): c for c in df.columns}
    
    for display_name, key, possible_cols in AVAILABLE_ITEMS_CONFIG:
        # Check if any of the possible columns exist
        found = False
        for col in possible_cols:
            if col in df.columns:
                found = True
                break
            # Case-insensitive check
            if col.lower() in columns_lower:
                found = True
                break
        if found:
            available.append((display_name, key))
    
    return available if available else [(name, key) for name, key, _ in AVAILABLE_ITEMS_CONFIG]


class ProductionOverTimePanel(BasePanel):
    """Panel for analyzing production over time of top items."""
    
    title = "Production Over Time"
    icon = "📅"
    description = "Analyze production over time of top authors, sources, keywords, countries"
    requires_data = True
    
    def __init__(self, parent, theme: str = "light", bib=None, **kwargs):
        self._result_df = None
        self._current_fig = None
        super().__init__(parent, theme=theme, bib=bib, **kwargs)
        self._primary_action = self._run_analysis
    
    def _create_options(self):
        """Create the options panel."""
        self._add_title()
        
        # Item Selection Card
        item_card = Card(self.options_content, title="📊 Item Selection", theme=self.theme_name)
        item_card.pack(fill=tk.X, padx=8, pady=8)
        
        # Get available items based on current data
        available_items = self._get_available_items()
        item_values = [f[0] for f in available_items]
        
        self.item_type = LabeledCombobox(
            item_card.content, label="Analyze:",
            values=item_values,
            default=item_values[0] if item_values else "Authors",
            theme=self.theme_name, label_width=10,
        )
        self.item_type.pack(fill=tk.X, pady=4)
        
        # Settings Card
        settings_card = Card(self.options_content, title="⚙️ Settings", theme=self.theme_name)
        settings_card.pack(fill=tk.X, padx=8, pady=8)
        
        self.top_n_spin = LabeledSpinbox(
            settings_card.content, label="Top N:",
            from_=5, to=50, default=10,
            theme=self.theme_name, label_width=12,
        )
        self.top_n_spin.pack(fill=tk.X, pady=4)
        
        self.min_docs_spin = LabeledSpinbox(
            settings_card.content, label="Min Documents:",
            from_=1, to=20, default=1,
            theme=self.theme_name, label_width=12,
        )
        self.min_docs_spin.pack(fill=tk.X, pady=4)
        
        # Plot Type Card
        plot_type_card = Card(self.options_content, title="📈 Plot Type", theme=self.theme_name)
        plot_type_card.pack(fill=tk.X, padx=8, pady=8)
        
        self.plot_type = LabeledCombobox(
            plot_type_card.content, label="View:",
            values=["Per Year", "Cumulative"],
            default="Per Year",
            theme=self.theme_name, label_width=12,
        )
        self.plot_type.pack(fill=tk.X, pady=4)
        
        # Plot Options Card
        plot_card = Card(self.options_content, title="🎨 Plot Options", theme=self.theme_name)
        plot_card.pack(fill=tk.X, padx=8, pady=8)
        
        self.cmap = LabeledCombobox(
            plot_card.content, label="Color Map:",
            values=["tab10", "Set1", "Set2", "Dark2", "Paired", "viridis", "plasma"],
            default="tab10",
            theme=self.theme_name, label_width=12,
        )
        self.cmap.pack(fill=tk.X, pady=4)
        self.cmap.pack(fill=tk.X, pady=4)
        
        # Run button
        # LLM Configuration (v2.11)
        self._create_llm_config_card()
        
        btn_frame = tk.Frame(self.options_content, bg=self.theme["bg_primary"])
        btn_frame.pack(fill=tk.X, padx=8, pady=16)
        
        self.run_btn = ActionButton(
            btn_frame, text="Run Analysis",
            command=self._run_analysis,
            icon="▶",
            theme=self.theme_name,
        )
        self.run_btn.pack(fill=tk.X)
        
        # Export buttons
        self.export_plot_btn = ThemedButton(
            btn_frame, text="Export Plot",
            command=self._export_plot,
            style="secondary",
            icon="🖼️",
            theme=self.theme_name,
        )
        self.export_plot_btn.pack(fill=tk.X, pady=(8, 0))
        
        self.export_data_btn = ThemedButton(
            btn_frame, text="Export Data",
            command=self._export_data,
            style="secondary",
            icon="📊",
            theme=self.theme_name,
        )
        self.export_data_btn.pack(fill=tk.X, pady=(8, 0))
    
    def _create_results(self):
        """Create the results panel."""
        # Create notebook for tabs
        self.notebook = ttk.Notebook(self.results_frame)
        self.notebook.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
        
        # Plot tab
        self.plot_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])
        self.notebook.add(self.plot_frame, text="📈 Plot")
        
        # Data tab
        self.data_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])
        self.notebook.add(self.data_frame, text="📋 Data")
        
        # Info tab
        self.info_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])
        self.notebook.add(self.info_frame, text="ℹ️ Info")
        
        # Info tab
        info_frame = tk.Frame(self.notebook, bg=self.theme["bg_card"])
        self.notebook.add(info_frame, text="ℹ️ Info")
        self._create_info_content(info_frame)
        
        # Initialize with placeholder
        self._show_placeholder()
        self._create_info_tab()
    
    def _show_placeholder(self):
        """Show placeholder message."""
        for widget in self.plot_frame.winfo_children():
            widget.destroy()
        
        tk.Label(
            self.plot_frame,
            text="Select an item type and click 'Run Analysis'\nto see production over time",
            font=FONTS.get_font("body"),
            bg=self.theme["bg_card"],
            fg=self.theme["text_secondary"],
            justify=tk.CENTER,
        ).pack(expand=True)
    
    def _create_info_tab(self):
        """Create the info tab."""
        for widget in self.info_frame.winfo_children():
            widget.destroy()
        
        info_text = """
PRODUCTION OVER TIME ANALYSIS
═════════════════════════════

This analysis shows the publication activity of top items (authors, 
sources, keywords, countries) over time.

VISUALIZATION
─────────────
• Each row represents one item (author, source, keyword, etc.)
• Horizontal lines show the publication span (first to last year)
• Circle size indicates number of documents in that year
• Circle color represents the selected metric (citations, h-index)

METRICS COMPUTED
────────────────
Per item, per year:
• Number of documents
• Total citations
• Citations per document

Per item (overall):
• Total documents
• Total citations
• H-index
• First and last publication year

AVAILABLE ITEMS
───────────────
• Authors: Individual author production patterns
• Author Keywords: Author-assigned keyword trends over time
• Index Keywords: Database-assigned keyword trends (Keywords Plus)
• Sources: Journal/venue publication patterns
• Countries: Geographic publication distribution
• Affiliations: Institutional production patterns
• Document Types: Distribution of article, review, conference paper, etc.
• References: Most cited references over time
• Cited Sources: Most cited journals in references
• Cited Authors: Most cited authors in references
• Research Areas: Subject area trends
• Subject Categories: WoS category distribution

INTERPRETATION
──────────────
• Rising lines indicate growth in output
• Steep slopes show rapid productivity increases
• Flat lines indicate stable production
• Gaps may indicate publication pauses

PLOT TYPES
──────────
• Per Year: Shows yearly document count as lines
• Cumulative: Shows accumulated output over time

SETTINGS
────────
• Top N: Number of top items to analyze (by total documents)
• Min Documents: Minimum documents required per item
• View: Choose between Per Year or Cumulative view
• Color Map: Color scheme for lines (tab10, Set1 best for distinct colors)
"""
        
        text_widget = tk.Text(
            self.info_frame,
            font=FONTS.get_font("body"),
            bg=self.theme["bg_card"],
            fg=self.theme["text_primary"],
            wrap=tk.WORD,
            padx=16, pady=16,
            relief=tk.FLAT,
        )
        text_widget.insert("1.0", info_text)
        text_widget.config(state=tk.DISABLED)
        text_widget.pack(fill=tk.BOTH, expand=True)
    
    def _get_available_items(self):
        """Get list of available items based on current data."""
        df = self.bib.df if self.bib is not None else None
        return get_available_items_for_df(df)
    
    def _get_item_key(self, display_name: str) -> str:
        """Convert display name to item key."""
        for name, key, _ in AVAILABLE_ITEMS_CONFIG:
            if name == display_name:
                return key
        return display_name.lower()
    
    def _run_analysis(self):
        """Run the production over time analysis."""
        if self.bib is None:
            messagebox.showwarning("No Data", "Please load a dataset first.")
            return
        
        self.run_btn.config(state=tk.DISABLED, text="⏳ Analyzing...")
        
        # Capture values before thread
        item_key = self._get_item_key(self.item_type.get())
        top_n = self.top_n_spin.get()
        min_docs = self.min_docs_spin.get()
        cmap = self.cmap.get()
        plot_type = self.plot_type.get()
        
        def do_analysis():
            error_info = None
            try:
                from biblium import utilsbib, plotbib
                
                # Build column mapping from config
                col_mapping = {key: cols for _, key, cols in AVAILABLE_ITEMS_CONFIG}
                
                # Find the actual column
                possible_cols = col_mapping.get(item_key, [item_key])
                actual_col = None
                for col in possible_cols:
                    if col in self.bib.df.columns:
                        actual_col = col
                        break
                
                if actual_col is None:
                    # Try case-insensitive match
                    for col in self.bib.df.columns:
                        col_lower = col.lower()
                        item_lower = item_key.replace("_", " ").lower()
                        if col_lower == item_lower or col_lower.replace(" ", "") == item_lower.replace(" ", ""):
                            actual_col = col
                            break
                
                if actual_col is None:
                    raise ValueError(f"Could not find column for '{item_key}' in dataset.")
                
                # Compute using utilsbib
                separator = getattr(self.bib, 'default_separator', '; ')
                if hasattr(self.bib, 'db') and self.bib.db == 'oa':
                    separator = r"\|"
                
                result_df = utilsbib.compute_item_time_stats(
                    self.bib.df, 
                    actual_col,
                    list_separators=separator,
                    top_n=top_n,
                    min_docs_per_item=min_docs,
                )
                
                # Create line chart based on plot_type
                if plot_type == "Cumulative":
                    fig = self._create_cumulative_plot(result_df, actual_col, cmap)
                else:
                    # Per Year view
                    fig = self._create_per_year_plot(result_df, actual_col, cmap)
                
                self._result_df = result_df
                self._current_fig = fig
                
                self.after(0, self._on_analysis_success)
                
            except Exception as exc:
                import traceback
                error_info = (str(exc), traceback.format_exc())
                self.after(0, lambda ei=error_info: self._on_analysis_error(ei[0], ei[1]))
        
        threading.Thread(target=do_analysis, daemon=True).start()
    
    def _create_per_year_plot(self, result_df, y_label, cmap):
        """Create per-year line chart for entities."""
        import matplotlib
        matplotlib.use('Agg')  # Use non-interactive backend
        from matplotlib.figure import Figure
        import numpy as np
        
        fig = Figure(figsize=(12, 6))
        ax = fig.add_subplot(111)
        
        # Find columns
        item_col = result_df.columns[0]
        year_col = None
        docs_col = None
        
        for col in ['Year', 'year', 'Publication Year']:
            if col in result_df.columns:
                year_col = col
                break
        
        for col in ['n_docs', 'Number of documents', 'Documents']:
            if col in result_df.columns:
                docs_col = col
                break
        
        if year_col is None or docs_col is None:
            raise ValueError(f"Required columns not found. Available: {list(result_df.columns)}")
        
        # Get unique items
        items = result_df[item_col].unique()
        
        # Get colormap
        import matplotlib.pyplot as plt
        if cmap in ['tab10', 'Set1', 'Set2', 'Set3', 'Paired', 'Accent', 'Dark2']:
            colors = plt.cm.get_cmap(cmap).colors[:len(items)]
        else:
            colors = plt.cm.get_cmap(cmap)(np.linspace(0.1, 0.9, len(items)))
        
        # Plot line for each item
        for idx, item in enumerate(items):
            item_data = result_df[result_df[item_col] == item].copy()
            if item_data.empty:
                continue
            
            item_data = item_data.sort_values(year_col)
            
            ax.plot(
                item_data[year_col], 
                item_data[docs_col],
                color=colors[idx % len(colors)],
                linewidth=2,
                marker='o',
                markersize=4,
                label=str(item)[:30] + ('...' if len(str(item)) > 30 else ''),
            )
        
        ax.set_xlabel('Year', fontsize=10)
        ax.set_ylabel(f'Documents ({y_label})', fontsize=10)
        ax.set_title('Entity Production Per Year', fontsize=12)
        ax.tick_params(axis='both', labelsize=8)
        ax.grid(False)
        ax.legend(loc='upper left', fontsize=8, framealpha=0.9)
        
        fig.tight_layout()
        return fig
    
    def _create_cumulative_plot(self, result_df, y_label, cmap):
        """Create cumulative growth plot for entities."""
        import matplotlib
        matplotlib.use('Agg')  # Use non-interactive backend
        from matplotlib.figure import Figure
        import matplotlib.pyplot as plt
        import numpy as np
        
        fig = Figure(figsize=(12, 6))
        ax = fig.add_subplot(111)
        
        # Find columns
        item_col = result_df.columns[0]
        year_col = None
        docs_col = None
        
        for col in ['Year', 'year', 'Publication Year']:
            if col in result_df.columns:
                year_col = col
                break
        
        for col in ['n_docs', 'Number of documents', 'Documents']:
            if col in result_df.columns:
                docs_col = col
                break
        
        if year_col is None or docs_col is None:
            raise ValueError(f"Required columns not found. Available: {list(result_df.columns)}")
        
        # Get unique items
        items = result_df[item_col].unique()
        
        # Get colormap - use qualitative colormaps for distinct colors
        if cmap in ['tab10', 'Set1', 'Set2', 'Set3', 'Paired', 'Accent']:
            colors = plt.cm.get_cmap(cmap).colors[:len(items)]
        else:
            colors = plt.cm.get_cmap(cmap)(np.linspace(0.1, 0.9, len(items)))
        
        # Plot cumulative line for each item
        for idx, item in enumerate(items):
            item_data = result_df[result_df[item_col] == item].copy()
            if item_data.empty:
                continue
            
            # Sort by year and compute cumulative
            item_data = item_data.sort_values(year_col)
            item_data['cumulative'] = item_data[docs_col].cumsum()
            
            # Plot
            ax.plot(
                item_data[year_col], 
                item_data['cumulative'],
                color=colors[idx % len(colors)],
                linewidth=2,
                marker='o',
                markersize=4,
                label=str(item)[:30] + ('...' if len(str(item)) > 30 else ''),
            )
        
        ax.set_xlabel('Year', fontsize=10)
        ax.set_ylabel(f'Cumulative Documents ({y_label})', fontsize=10)
        ax.set_title('Cumulative Growth Over Time', fontsize=12)
        ax.tick_params(axis='both', labelsize=8)
        
        # No gridlines
        ax.grid(False)
        
        # Legend in upper left corner
        ax.legend(loc='upper left', fontsize=8, framealpha=0.9)
        
        fig.tight_layout()
        return fig
    
    def _on_analysis_success(self):
        """Handle successful analysis."""
        self.run_btn.config(state=tk.NORMAL, text="▶ Run Analysis")
        self._update_plot_tab()
        self._update_data_tab()
        self.notebook.select(0)
    
    def _on_analysis_error(self, error_msg: str, traceback_str: str):
        """Handle analysis error."""
        self.run_btn.config(state=tk.NORMAL, text="▶ Run Analysis")
        messagebox.showerror("Analysis Error", f"Error: {error_msg}\n\n{traceback_str[:500]}")
    
    def _update_plot_tab(self):
        """Update the plot tab."""
        for widget in self.plot_frame.winfo_children():
            widget.destroy()
        
        if self._current_fig is None:
            tk.Label(
                self.plot_frame,
                text="No plot available",
                font=FONTS.get_font("body"),
                bg=self.theme["bg_card"],
                fg=self.theme["danger"],
            ).pack(expand=True)
            return
        
        try:
            from biblium.gui.widgets.plots import ScaledImageFrame
            
            scaled_frame = ScaledImageFrame(
                self.plot_frame, 
                theme=self.theme_name,
                maintain_aspect=True,
                max_scale=1.5
            )
            scaled_frame.pack(fill=tk.BOTH, expand=True)
            scaled_frame.set_image_from_figure(self._current_fig, dpi=100)
            
            self._scaled_frame = scaled_frame
            self._add_save_menu(scaled_frame)
            
        except Exception as e:
            tk.Label(
                self.plot_frame,
                text=f"Plot generated successfully.\nUse 'Export Plot' to save.\n\n(Display error: {e})",
                font=FONTS.get_font("body"),
                bg=self.theme["bg_card"],
                fg=self.theme["text_secondary"],
                wraplength=400,
            ).pack(expand=True, padx=20, pady=20)
    
    def _add_save_menu(self, widget):
        """Add right-click context menu for saving."""
        def show_menu(event):
            menu = tk.Menu(widget, tearoff=0)
            menu.add_command(label="📄 Add to Report", command=self._add_plot_to_report)
            menu.add_separator()
            menu.add_command(label="💾 Save as PNG...", command=lambda: self._save_as("png"))
            menu.add_command(label="💾 Save as PDF...", command=lambda: self._save_as("pdf"))
            menu.add_command(label="💾 Save as SVG...", command=lambda: self._save_as("svg"))
            menu.tk_popup(event.x_root, event.y_root)
        
        widget.bind("<Button-3>", show_menu)
    
    def _add_plot_to_report(self):
        """Add current plot to report queue."""
        if self._current_fig is None:
            messagebox.showinfo("No Plot", "No plot to add to report.")
            return
        
        try:
            from biblium.gui.core.state import report_queue
            
            plot_title = "Production Analysis"
            if self._current_fig.axes:
                plot_title = self._current_fig.axes[0].get_title() or "Production Analysis"
            
            report_queue.add_plot(
                figure_or_bytes=self._current_fig,
                title=plot_title,
                source_panel=self.title,
            )
            
            messagebox.showinfo(
                "Added to Report",
                f"Plot '{plot_title}' has been added to the report queue.\n\n"
                f"Items in queue: {len(report_queue)}\n\n"
                "Go to Reports → Report Builder to generate your report."
            )
        except Exception as e:
            messagebox.showerror("Error", f"Failed to add to report: {e}")
    
    def _save_as(self, fmt):
        """Save figure in specified format."""
        if self._current_fig is None:
            return
        
        filepath = filedialog.asksaveasfilename(
            defaultextension=f".{fmt}",
            filetypes=[(f"{fmt.upper()} files", f"*.{fmt}")],
            title=f"Save as {fmt.upper()}",
        )
        if filepath:
            try:
                self._current_fig.savefig(filepath, dpi=300, bbox_inches='tight')
                messagebox.showinfo("Saved", f"Plot saved to:\n{filepath}")
            except Exception as e:
                messagebox.showerror("Error", f"Failed to save: {e}")
    
    def _update_data_tab(self):
        """Update the data table tab."""
        for widget in self.data_frame.winfo_children():
            widget.destroy()
        
        if self._result_df is None or self._result_df.empty:
            tk.Label(
                self.data_frame,
                text="No data available",
                font=FONTS.get_font("body"),
                bg=self.theme["bg_card"],
                fg=self.theme["text_secondary"],
            ).pack(expand=True)
            return
        
        # Format the dataframe for display
        df = self._result_df.copy()
        
        # Round numeric columns
        for col in df.select_dtypes(include=[np.number]).columns:
            if "citation" in col.lower() or "per" in col.lower():
                df[col] = df[col].round(2)
            else:
                df[col] = df[col].round(0).astype(int, errors='ignore')
        
        # Create data table
        table = DataTable(
            self.data_frame,
            theme=self.theme_name,
            show_index=False,
        )
        table.set_data(df)
        table.pack(fill=tk.BOTH, expand=True, padx=8, pady=8)
    
    def _export_plot(self):
        """Export the plot."""
        if self._current_fig is None:
            messagebox.showwarning("No Plot", "Please run analysis first.")
            return
        
        filepath = filedialog.asksaveasfilename(
            defaultextension=".png",
            filetypes=[
                ("PNG files", "*.png"),
                ("PDF files", "*.pdf"),
                ("SVG files", "*.svg"),
            ],
            title="Export Plot",
        )
        
        if not filepath:
            return
        
        try:
            self._current_fig.savefig(filepath, dpi=300, bbox_inches='tight')
            messagebox.showinfo("Export Complete", f"Plot exported to:\n{filepath}")
        except Exception as e:
            messagebox.showerror("Export Error", f"Failed to export: {e}")
    
    def _export_data(self):
        """Export the data table."""
        if self._result_df is None or self._result_df.empty:
            messagebox.showwarning("No Data", "Please run analysis first.")
            return
        
        filepath = filedialog.asksaveasfilename(
            defaultextension=".xlsx",
            filetypes=[
                ("Excel files", "*.xlsx"),
                ("CSV files", "*.csv"),
            ],
            title="Export Data",
        )
        
        if not filepath:
            return
        
        try:
            if filepath.endswith(".xlsx"):
                self._result_df.to_excel(filepath, index=False)
            else:
                self._result_df.to_csv(filepath, index=False)
            messagebox.showinfo("Export Complete", f"Data exported to:\n{filepath}")
        except Exception as e:
            messagebox.showerror("Export Error", f"Failed to export: {e}")
    
    def on_data_loaded(self, bib):
        """Handle data loaded event."""
        self.bib = bib
        self._cleanup_figure()
        self._result_df = None
        self._show_placeholder()
        
        # Update available items in dropdown
        self._update_available_items()
    
    def _update_available_items(self):
        """Update the item dropdown based on available columns."""
        if hasattr(self, 'item_type'):
            available_items = self._get_available_items()
            item_values = [f[0] for f in available_items]
            
            # Update combobox values
            self.item_type.combo['values'] = item_values
            
            # Keep current selection if still valid, otherwise reset
            current = self.item_type.get()
            if current not in item_values and item_values:
                self.item_type.combo.set(item_values[0])
    
    def _cleanup_figure(self):
        """Clean up matplotlib figure to prevent threading issues."""
        if self._current_fig is not None:
            try:
                import matplotlib.pyplot as plt
                plt.close(self._current_fig)
            except:
                pass
            self._current_fig = None
        
        # Clear canvas reference
        if hasattr(self, '_canvas'):
            self._canvas = None
        if hasattr(self, '_toolbar'):
            self._toolbar = None
    
    def _create_info_content(self, parent):
        """Create Info tab with documentation."""
        info_text = """
SCIENTIFIC PRODUCTION
═════════════════════

Analyze publication output patterns.

METRICS
───────
• Total documents
• Annual output
• Growth rate
• Author productivity

AUTHOR ANALYSIS
───────────────
• Most productive authors
• Productivity distribution
• Lotka's Law fit

SOURCE ANALYSIS
───────────────
• Core journals
• Bradford zones
• Source productivity

TEMPORAL PATTERNS
─────────────────
• Publication trends
• Growth modeling
• Period comparisons
"""
        text_widget = tk.Text(
            parent, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            wrap=tk.WORD, padx=16, pady=16, relief=tk.FLAT,
        )
        text_widget.insert("1.0", info_text)
        text_widget.config(state=tk.DISABLED)
        text_widget.pack(fill=tk.BOTH, expand=True)
        
        # Add right-click context menu for copy
        def show_context_menu(event):
            menu = tk.Menu(text_widget, tearoff=0)
            menu.add_command(label="Copy Selected", command=lambda: copy_selected(text_widget))
            menu.add_command(label="Copy All", command=lambda: copy_all(text_widget))
            menu.tk_popup(event.x_root, event.y_root)
        
        def copy_selected(widget):
            try:
                widget.config(state=tk.NORMAL)
                selected = widget.get(tk.SEL_FIRST, tk.SEL_LAST)
                widget.config(state=tk.DISABLED)
                widget.clipboard_clear()
                widget.clipboard_append(selected)
            except tk.TclError:
                pass  # No selection
        
        def copy_all(widget):
            widget.config(state=tk.NORMAL)
            content = widget.get("1.0", tk.END)
            widget.config(state=tk.DISABLED)
            widget.clipboard_clear()
            widget.clipboard_append(content.strip())
        
        text_widget.bind("<Button-3>", show_context_menu)  # Right-click
        text_widget.bind("<Control-c>", lambda e: copy_selected(text_widget))

    def destroy(self):
        """Clean up resources before destroying widget."""
        self._cleanup_figure()
        super().destroy()



# -*- coding: utf-8 -*-
"""
Topic Modeling Utilities.

This module provides functions for topic modeling and conceptual structure analysis:
- LDA (Latent Dirichlet Allocation)
- NMF (Non-negative Matrix Factorization)
- LSA (Latent Semantic Analysis)
- Conceptual Structure Analysis (factor analysis, clustering, MDS/PCA)
"""

from __future__ import annotations

from typing import Any, Dict, List, Literal, Optional, Tuple, Union

import numpy as np
import pandas as pd


# Lazy imports
def _get_vectorizers():
    """Get sklearn vectorizers (lazy import)."""
    from sklearn.feature_extraction.text import CountVectorizer, TfidfVectorizer
    return CountVectorizer, TfidfVectorizer


def _get_decomposition():
    """Get sklearn decomposition classes (lazy import)."""
    from sklearn.decomposition import LatentDirichletAllocation, NMF, TruncatedSVD
    return LatentDirichletAllocation, NMF, TruncatedSVD


def _get_clustering():
    """Get sklearn clustering classes (lazy import)."""
    from sklearn.cluster import KMeans, AgglomerativeClustering
    return KMeans, AgglomerativeClustering


def _get_manifold():
    """Get sklearn manifold classes (lazy import)."""
    from sklearn.manifold import MDS
    from sklearn.decomposition import PCA
    return MDS, PCA


def topic_modeling(
    df: pd.DataFrame,
    text_column: str,
    model_type: str = "LDA",
    n_topics: Optional[int] = None,
    max_topics: int = 10,
    max_features: int = 5000,
    stop_words: Union[str, List[str], None] = "english",
    *,
    include_doc_topic_weights: bool = True,
    topic_prefix: str = "Topic",
    weight_suffix: str = "Weight",
    normalize: Literal["row_sum", "softmax", "none"] = "row_sum",
    lsa_nonneg: Literal["abs", "square", "none"] = "abs",
    empty_weight_as_nan: bool = True,
) -> Tuple[pd.DataFrame, pd.DataFrame]:
    """
    Perform topic modeling (LDA, NMF, or LSA) on text data.
    
    Parameters
    ----------
    df : pd.DataFrame
        Input DataFrame with text data.
    text_column : str
        Column containing text to model.
    model_type : str
        Type of topic model: 'LDA', 'NMF', or 'LSA'.
    n_topics : int, optional
        Number of topics. If None, uses coherence-based selection.
    max_topics : int
        Maximum number of topics to consider.
    max_features : int
        Maximum vocabulary size.
    stop_words : str, list, or None
        Stop words to remove. 'english' for default list.
    include_doc_topic_weights : bool
        If True, add topic weight columns to output.
    topic_prefix : str
        Prefix for topic columns.
    weight_suffix : str
        Suffix for weight columns.
    normalize : str
        Normalization method for weights: 'row_sum', 'softmax', 'none'.
    lsa_nonneg : str
        How to handle negative LSA values: 'abs', 'square', 'none'.
    empty_weight_as_nan : bool
        If True, empty texts get NaN weights.
        
    Returns
    -------
    tuple
        (df_with_topics, topics_df)
        - df_with_topics: Original DataFrame with Topic column and optionally weight columns
        - topics_df: DataFrame with Topic, Term, Weight columns for top terms
    """
    CountVectorizer, TfidfVectorizer = _get_vectorizers()
    LDA, NMF, TruncatedSVD = _get_decomposition()
    
    if text_column not in df.columns:
        raise KeyError(f"Column '{text_column}' not found in df.")
    
    texts = df[text_column].fillna("").astype(str).str.strip()
    mask = texts != ""
    df_out = df.copy()
    
    # Early return if nothing to model
    if mask.sum() == 0:
        df_out["Topic"] = pd.Series([pd.NA] * len(df_out), index=df_out.index, dtype="Float64")
        return df_out, pd.DataFrame(columns=["Topic", "Term", "Weight"])
    
    # Select vectorizer based on model type
    mt = model_type.upper()
    if mt in {"LDA", "LSA"}:
        vectorizer = CountVectorizer(max_features=max_features, stop_words=stop_words)
    elif mt == "NMF":
        vectorizer = TfidfVectorizer(max_features=max_features, stop_words=stop_words)
    else:
        raise ValueError('model_type must be one of: "LDA", "NMF", "LSA".')
    
    # Vectorize valid rows
    try:
        X = vectorizer.fit_transform(texts[mask])
    except ValueError:
        df_out["Topic"] = pd.Series([pd.NA] * len(df_out), index=df_out.index, dtype="Float64")
        return df_out, pd.DataFrame(columns=["Topic", "Term", "Weight"])
    
    # Determine number of topics
    if n_topics is None:
        n_topics = min(max_topics, X.shape[1], mask.sum())
    n_topics = max(1, min(n_topics, X.shape[1], mask.sum()))
    
    # Build model
    if mt == "LDA":
        model = LDA(n_components=n_topics, random_state=42)
        doc_topic = model.fit_transform(X)
    elif mt == "NMF":
        model = NMF(n_components=n_topics, random_state=42, init="nndsvd")
        doc_topic = model.fit_transform(X)
    else:  # LSA
        model = TruncatedSVD(n_components=n_topics, random_state=42)
        doc_topic = model.fit_transform(X)
        
        # Handle negative values
        if lsa_nonneg == "abs":
            doc_topic = np.abs(doc_topic)
        elif lsa_nonneg == "square":
            doc_topic = doc_topic ** 2
    
    # Normalize if requested
    if normalize == "row_sum":
        row_sums = doc_topic.sum(axis=1, keepdims=True)
        row_sums[row_sums == 0] = 1
        doc_topic = doc_topic / row_sums
    elif normalize == "softmax":
        exp_vals = np.exp(doc_topic - doc_topic.max(axis=1, keepdims=True))
        doc_topic = exp_vals / exp_vals.sum(axis=1, keepdims=True)
    
    # Assign dominant topic
    topic_assignments = doc_topic.argmax(axis=1) + 1  # 1-indexed
    
    # Create topic column
    topic_col = pd.Series([pd.NA] * len(df_out), index=df_out.index, dtype="Float64")
    topic_col.loc[mask] = topic_assignments
    df_out["Topic"] = topic_col
    
    # Add weight columns if requested
    if include_doc_topic_weights:
        weight_matrix = np.full((len(df_out), n_topics), np.nan if empty_weight_as_nan else 0.0)
        weight_matrix[mask.values] = doc_topic
        
        for i in range(n_topics):
            col_name = f"{topic_prefix} {i+1} {weight_suffix}"
            df_out[col_name] = weight_matrix[:, i]
    
    # Extract top terms per topic
    feature_names = vectorizer.get_feature_names_out()
    if mt == "LSA":
        components = np.abs(model.components_)
    else:
        components = model.components_
    
    topics_data = []
    for topic_idx in range(n_topics):
        top_indices = components[topic_idx].argsort()[-10:][::-1]
        for idx in top_indices:
            topics_data.append({
                "Topic": topic_idx + 1,
                "Term": feature_names[idx],
                "Weight": components[topic_idx, idx],
            })
    
    topics_df = pd.DataFrame(topics_data)
    
    return df_out, topics_df


def get_topic_summary(
    topics_df: pd.DataFrame,
    n_terms: int = 10,
) -> Dict[int, List[str]]:
    """
    Get summary of topics with top terms.
    
    Parameters
    ----------
    topics_df : pd.DataFrame
        Topic terms DataFrame from topic_modeling().
    n_terms : int
        Number of terms per topic.
        
    Returns
    -------
    dict
        Mapping from topic number to list of top terms.
    """
    summary = {}
    for topic in topics_df["Topic"].unique():
        terms = (
            topics_df[topics_df["Topic"] == topic]
            .nlargest(n_terms, "Weight")["Term"]
            .tolist()
        )
        summary[int(topic)] = terms
    return summary


def compute_topic_coherence(
    df: pd.DataFrame,
    text_column: str,
    topics_df: pd.DataFrame,
    n_terms: int = 10,
) -> Dict[int, float]:
    """
    Compute topic coherence scores using co-occurrence.
    
    Parameters
    ----------
    df : pd.DataFrame
        Original DataFrame with text.
    text_column : str
        Column containing text.
    topics_df : pd.DataFrame
        Topic terms DataFrame.
    n_terms : int
        Number of terms to use for coherence.
        
    Returns
    -------
    dict
        Coherence score per topic.
    """
    texts = df[text_column].fillna("").str.lower()
    
    coherence = {}
    for topic in topics_df["Topic"].unique():
        terms = (
            topics_df[topics_df["Topic"] == topic]
            .nlargest(n_terms, "Weight")["Term"]
            .str.lower()
            .tolist()
        )
        
        # Count co-occurrences
        cooc_sum = 0
        pair_count = 0
        
        for i, term1 in enumerate(terms):
            for term2 in terms[i+1:]:
                # Count documents containing both terms
                both = ((texts.str.contains(term1, regex=False)) & 
                       (texts.str.contains(term2, regex=False))).sum()
                # Count documents containing term1
                single = texts.str.contains(term1, regex=False).sum()
                
                if single > 0:
                    cooc_sum += np.log((both + 1) / single)
                    pair_count += 1
        
        coherence[int(topic)] = cooc_sum / pair_count if pair_count > 0 else 0.0
    
    return coherence


def label_topics_with_llm(
    topics_df: pd.DataFrame,
    api_key: Optional[str] = None,
    model: str = "gpt-3.5-turbo",
    n_terms: int = 10,
) -> Dict[int, str]:
    """
    Generate topic labels using an LLM.
    
    Parameters
    ----------
    topics_df : pd.DataFrame
        Topic terms DataFrame.
    api_key : str, optional
        OpenAI API key. If None, uses OPENAI_API_KEY environment variable.
    model : str
        Model to use.
    n_terms : int
        Number of terms to include in prompt.
        
    Returns
    -------
    dict
        Mapping from topic number to generated label.
    """
    import os
    
    api_key = api_key or os.environ.get("OPENAI_API_KEY")
    if not api_key:
        raise ValueError("API key required. Set OPENAI_API_KEY or pass api_key.")
    
    try:
        from openai import OpenAI
    except ImportError:
        raise ImportError("openai package required: pip install openai")
    
    client = OpenAI(api_key=api_key)
    
    labels = {}
    for topic in topics_df["Topic"].unique():
        terms = (
            topics_df[topics_df["Topic"] == topic]
            .nlargest(n_terms, "Weight")["Term"]
            .tolist()
        )
        
        prompt = f"""Given these topic terms from a bibliometric analysis:
{', '.join(terms)}

Provide a short (2-4 word) descriptive label for this topic.
Return only the label, nothing else."""
        
        response = client.chat.completions.create(
            model=model,
            messages=[{"role": "user", "content": prompt}],
            max_tokens=20,
            temperature=0.3,
        )
        
        labels[int(topic)] = response.choices[0].message.content.strip()
    
    return labels



# -*- coding: utf-8 -*-
"""
Form Widgets
============
Input widgets with labels and consistent styling.
"""

import tkinter as tk
from tkinter import ttk
from typing import Any, Callable, List, Optional, Tuple, Union

from biblium.gui.config import FONTS, LAYOUT, get_theme


class LabeledEntry(tk.Frame):
    """
    Entry field with label.
    """
    
    def __init__(
        self,
        parent,
        label: str,
        default: str = "",
        placeholder: str = "",
        variable: Optional[tk.StringVar] = None,
        width: int = 25,
        theme: str = "light",
        tooltip: str = "",
        label_width: int = 20,
        **kwargs
    ):
        self.theme = get_theme(theme)
        self.placeholder = placeholder
        self._has_placeholder = False
        
        # Remove variable from kwargs if accidentally passed
        kwargs.pop('variable', None)
        
        super().__init__(parent, bg=self.theme["bg_card"], **kwargs)
        
        self.label = tk.Label(
            self, text=label, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            width=label_width, anchor=tk.W,
        )
        self.label.pack(side=tk.LEFT)
        
        # Use provided variable or create new one
        if variable is not None:
            self.var = variable
        else:
            self.var = tk.StringVar(value=default)
        
        self.entry = tk.Entry(
            self, textvariable=self.var, font=FONTS.get_font("body"),
            width=width, relief=tk.FLAT, bg=self.theme["bg_input"],
            fg=self.theme["text_primary"], insertbackground=self.theme["text_primary"],
            highlightbackground=self.theme["border"],
            highlightcolor=self.theme["border_focus"], highlightthickness=1,
            disabledforeground=self.theme["text_muted"],
        )
        self.entry.pack(side=tk.LEFT, fill=tk.X, expand=True, padx=(LAYOUT.spacing_sm, 0))
        
        if placeholder and not default:
            self._show_placeholder()
        
        self.entry.bind("<FocusIn>", self._on_focus_in)
        self.entry.bind("<FocusOut>", self._on_focus_out)
        self.entry.bind("<Key>", self._on_key)
        
        if tooltip:
            from biblium.gui.widgets.tooltips import create_tooltip
            create_tooltip(self.label, tooltip)
    
    def _show_placeholder(self):
        self.entry.configure(fg=self.theme["text_muted"])
        self.var.set(self.placeholder)
        self._has_placeholder = True
    
    def _hide_placeholder(self):
        self.entry.configure(fg=self.theme["text_primary"])
        self.var.set("")
        self._has_placeholder = False
    
    def _on_focus_in(self, event):
        if self._has_placeholder:
            self._hide_placeholder()
        # Always ensure text color is correct on focus
        self.entry.configure(fg=self.theme["text_primary"])
    
    def _on_focus_out(self, event):
        if not self.var.get() and self.placeholder:
            self._show_placeholder()
    
    def _on_key(self, event):
        """Ensure text is visible when typing."""
        if self._has_placeholder:
            self._hide_placeholder()
        # Ensure text color is always visible when typing
        self.entry.configure(fg=self.theme["text_primary"])
    
    def get(self) -> str:
        if self._has_placeholder:
            return ""
        return self.var.get()
    
    def set(self, value: str):
        if self._has_placeholder:
            self._hide_placeholder()
        self.var.set(value)
    
    def clear(self):
        self.var.set("")
        if self.placeholder:
            self._show_placeholder()


class LabeledCombobox(tk.Frame):
    """
    Combobox with label.
    """
    
    def __init__(
        self,
        parent,
        label: str,
        values: List[str],
        default: str = None,
        variable: Optional[tk.StringVar] = None,
        width: int = 22,
        theme: str = "light",
        tooltip: str = "",
        label_width: int = 20,
        command: Optional[Callable] = None,
        **kwargs
    ):
        self.theme = get_theme(theme)
        self._command = command
        
        # Remove variable from kwargs if accidentally passed
        kwargs.pop('variable', None)
        
        super().__init__(parent, bg=self.theme["bg_card"], **kwargs)
        
        self.label = tk.Label(
            self, text=label, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            width=label_width, anchor=tk.W,
        )
        self.label.pack(side=tk.LEFT)
        
        # Use provided variable or create new one
        if variable is not None:
            self.var = variable
        else:
            self.var = tk.StringVar()
        
        self.combo = ttk.Combobox(
            self, textvariable=self.var, values=values,
            state="readonly", width=width, font=FONTS.get_font("body"),
        )
        self.combo.pack(side=tk.LEFT, padx=(LAYOUT.spacing_sm, 0))
        
        # Alias for backwards compatibility
        self.combobox = self.combo
        
        if default and default in values:
            self.combo.set(default)
        elif variable is None and values:
            # Only set default if we created the variable
            self.combo.current(0)
        
        if command:
            self.combo.bind("<<ComboboxSelected>>", lambda e: command(self.var.get()))
        
        if tooltip:
            from biblium.gui.widgets.tooltips import create_tooltip
            create_tooltip(self.label, tooltip)
    
    def get(self) -> str:
        return self.var.get()
    
    def set(self, value: str):
        self.var.set(value)
    
    def get_index(self) -> int:
        return self.combo.current()
    
    def set_values(self, values: List[str], keep_selection: bool = True):
        current = self.get() if keep_selection else None
        self.combo["values"] = values
        if current and current in values:
            self.set(current)
        elif values:
            self.combo.current(0)


class LabeledCheckbox(tk.Frame):
    """
    Checkbox with label.
    """
    
    def __init__(
        self,
        parent,
        label: str,
        default: bool = False,
        variable: Optional[tk.BooleanVar] = None,
        theme: str = "light",
        tooltip: str = "",
        command: Optional[Callable] = None,
        **kwargs
    ):
        self.theme = get_theme(theme)
        self._command = command
        
        # Remove variable from kwargs if accidentally passed
        kwargs.pop('variable', None)
        
        super().__init__(parent, bg=self.theme["bg_card"], **kwargs)
        
        # Use provided variable or create new one
        if variable is not None:
            self.var = variable
        else:
            self.var = tk.BooleanVar(value=default)
        
        self.checkbox = tk.Checkbutton(
            self, text=label, variable=self.var,
            font=FONTS.get_font("body"), bg=self.theme["bg_card"],
            fg=self.theme["text_primary"], activebackground=self.theme["bg_card"],
            activeforeground=self.theme["text_primary"],
            selectcolor=self.theme["bg_input"], command=self._on_change,
        )
        self.checkbox.pack(side=tk.LEFT)
        
        if tooltip:
            from biblium.gui.widgets.tooltips import create_tooltip
            create_tooltip(self.checkbox, tooltip)
    
    def _on_change(self):
        if self._command:
            self._command(self.var.get())
    
    def get(self) -> bool:
        return self.var.get()
    
    def set(self, value: bool):
        self.var.set(value)


class LabeledSpinbox(tk.Frame):
    """
    Spinbox with label.
    """
    
    def __init__(
        self,
        parent,
        label: str,
        from_: int = 0,
        to: int = 100,
        default: int = None,
        variable: Optional[tk.IntVar] = None,
        width: int = 10,
        theme: str = "light",
        tooltip: str = "",
        label_width: int = 20,
        command: Optional[Callable] = None,
        **kwargs
    ):
        self.theme = get_theme(theme)
        self._command = command
        
        # Remove variable from kwargs if accidentally passed
        kwargs.pop('variable', None)
        
        super().__init__(parent, bg=self.theme["bg_card"], **kwargs)
        
        self.label = tk.Label(
            self, text=label, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"],
            width=label_width, anchor=tk.W,
        )
        self.label.pack(side=tk.LEFT)
        
        # Use provided variable or create new one
        if variable is not None:
            self.var = variable
        else:
            self.var = tk.IntVar(value=default if default is not None else from_)
        
        self.spinbox = tk.Spinbox(
            self, from_=from_, to=to, textvariable=self.var,
            font=FONTS.get_font("body"), width=width, relief=tk.FLAT,
            bg=self.theme["bg_input"], fg=self.theme["text_primary"],
            buttonbackground=self.theme["bg_secondary"],
            highlightbackground=self.theme["border"],
            highlightcolor=self.theme["border_focus"], highlightthickness=1,
            command=self._on_change,
        )
        self.spinbox.pack(side=tk.LEFT, padx=(LAYOUT.spacing_sm, 0))
        
        if tooltip:
            from biblium.gui.widgets.tooltips import create_tooltip
            create_tooltip(self.label, tooltip)
    
    def _on_change(self):
        if self._command:
            try:
                self._command(self.var.get())
            except tk.TclError:
                pass
    
    def get(self) -> int:
        try:
            return self.var.get()
        except tk.TclError:
            return 0
    
    def set(self, value: int):
        self.var.set(value)


class FormSection(tk.Frame):
    """
    A section of form fields with optional title.
    """
    
    def __init__(
        self,
        parent,
        title: str = "",
        theme: str = "light",
        **kwargs
    ):
        self.theme = get_theme(theme)
        
        super().__init__(parent, bg=self.theme["bg_card"], **kwargs)
        
        if title:
            title_label = tk.Label(
                self, text=title, font=FONTS.get_font("body", bold=True),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"], anchor=tk.W,
            )
            title_label.pack(fill=tk.X, pady=(0, LAYOUT.spacing_sm))
        
        self.content = tk.Frame(self, bg=self.theme["bg_card"])
        self.content.pack(fill=tk.BOTH, expand=True)
    
    def add_field(self, widget_class, **kwargs) -> Any:
        kwargs.setdefault("theme", self.theme.get("name", "light"))
        widget = widget_class(self.content, **kwargs)
        widget.pack(fill=tk.X, pady=LAYOUT.spacing_xs)
        return widget


class LabeledTextArea(tk.Frame):
    """
    Multi-line text area with label.
    """
    
    def __init__(
        self,
        parent,
        label: str,
        height: int = 4,
        width: int = 40,
        default: str = "",
        theme: str = "light",
        tooltip: str = "",
        **kwargs
    ):
        self.theme = get_theme(theme)
        
        super().__init__(parent, bg=self.theme["bg_card"], **kwargs)
        
        self.label = tk.Label(
            self, text=label, font=FONTS.get_font("body"),
            bg=self.theme["bg_card"], fg=self.theme["text_primary"], anchor=tk.W,
        )
        self.label.pack(fill=tk.X)
        
        text_frame = tk.Frame(self, bg=self.theme["border"])
        text_frame.pack(fill=tk.BOTH, expand=True, pady=(LAYOUT.spacing_xs, 0))
        
        self.text = tk.Text(
            text_frame, font=FONTS.get_font("body"), height=height, width=width,
            relief=tk.FLAT, bg=self.theme["bg_input"], fg=self.theme["text_primary"],
            insertbackground=self.theme["text_primary"],
            padx=LAYOUT.spacing_sm, pady=LAYOUT.spacing_sm,
        )
        self.text.pack(fill=tk.BOTH, expand=True, padx=1, pady=1)
        
        if default:
            self.text.insert("1.0", default)
        
        if tooltip:
            from biblium.gui.widgets.tooltips import create_tooltip
            create_tooltip(self.label, tooltip)
    
    def get(self) -> str:
        return self.text.get("1.0", tk.END).strip()
    
    def set(self, value: str):
        self.text.delete("1.0", tk.END)
        self.text.insert("1.0", value)
    
    def clear(self):
        self.text.delete("1.0", tk.END)


class RadioGroup(tk.Frame):
    """
    Group of radio buttons.
    """
    
    def __init__(
        self,
        parent,
        label: str,
        options: List[str],
        default: str = None,
        orientation: str = "horizontal",
        theme: str = "light",
        command: Optional[Callable] = None,
        **kwargs
    ):
        self.theme = get_theme(theme)
        self._command = command
        
        super().__init__(parent, bg=self.theme["bg_card"], **kwargs)
        
        if label:
            self.label = tk.Label(
                self, text=label, font=FONTS.get_font("body"),
                bg=self.theme["bg_card"], fg=self.theme["text_primary"], anchor=tk.W,
            )
            self.label.pack(fill=tk.X)
        
        options_frame = tk.Frame(self, bg=self.theme["bg_card"])
        options_frame.pack(fill=tk.X, pady=(LAYOUT.spacing_xs, 0))
        
        self.var = tk.StringVar(value=default or (options[0] if options else ""))
        
        for option in options:
            rb = tk.Radiobutton(
                options_frame, text=option, variable=self.var, value=option,
                font=FONTS.get_font("body"), bg=self.theme["bg_card"],
                fg=self.theme["text_primary"], activebackground=self.theme["bg_card"],
                activeforeground=self.theme["text_primary"],
                selectcolor=self.theme["bg_input"], command=self._on_change,
            )
            if orientation == "horizontal":
                rb.pack(side=tk.LEFT, padx=(0, LAYOUT.spacing_md))
            else:
                rb.pack(anchor=tk.W)
    
    def _on_change(self):
        if self._command:
            self._command(self.var.get())
    
    def get(self) -> str:
        return self.var.get()
    
    def set(self, value: str):
        self.var.set(value)


class DualListSelector(tk.Frame):
    """
    Two-column list selector with Available and Selected lists.
    Allows moving items between lists using arrow buttons.
    
    By default:
    - Left column (available_label): GREEN - items to include/show
    - Right column (selected_label): RED - items to exclude/hide
    """
    
    def __init__(
        self,
        parent,
        available_label: str = "Available",
        selected_label: str = "Selected",
        available_color: str = "#27ae60",  # Green
        selected_color: str = "#c0392b",   # Red
        height: int = 8,
        theme: str = "light",
        **kwargs
    ):
        self.theme = get_theme(theme)
        self.theme_name = theme
        self.available_color = available_color
        self.selected_color = selected_color
        
        super().__init__(parent, bg=self.theme["bg_card"], **kwargs)
        
        # Main container
        container = tk.Frame(self, bg=self.theme["bg_card"])
        container.pack(fill=tk.BOTH, expand=True)
        
        # Available list (left)
        available_frame = tk.Frame(container, bg=self.theme["bg_card"])
        available_frame.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        
        tk.Label(
            available_frame, text=available_label,
            font=FONTS.get_font("body", bold=True),
            bg=self.theme["bg_card"], fg=available_color,
        ).pack(anchor=tk.W)
        
        # Available listbox with scrollbar
        avail_list_frame = tk.Frame(available_frame, bg=self.theme["bg_card"])
        avail_list_frame.pack(fill=tk.BOTH, expand=True, pady=2)
        
        avail_scrollbar = ttk.Scrollbar(avail_list_frame, orient=tk.VERTICAL)
        avail_scrollbar.pack(side=tk.RIGHT, fill=tk.Y)
        
        self.available_listbox = tk.Listbox(
            avail_list_frame, height=height,
            font=FONTS.get_font("small"),
            selectmode=tk.EXTENDED,
            bg=self.theme["bg_input"],
            fg=self.theme["text_primary"],
            selectbackground=available_color,
            selectforeground="white",
            relief=tk.FLAT,
            highlightthickness=1,
            highlightbackground=self.theme["border"],
            yscrollcommand=avail_scrollbar.set,
            exportselection=False,
        )
        self.available_listbox.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        avail_scrollbar.config(command=self.available_listbox.yview)
        
        # Arrow buttons (center)
        btn_frame = tk.Frame(container, bg=self.theme["bg_card"])
        btn_frame.pack(side=tk.LEFT, padx=4)
        
        btn_style = {
            "font": FONTS.get_font("body", bold=True),
            "width": 3,
            "bg": self.theme["bg_input"],
            "fg": self.theme["text_primary"],
            "relief": tk.FLAT,
            "cursor": "hand2",
        }
        
        tk.Button(
            btn_frame, text="→", command=self._move_to_selected, **btn_style
        ).pack(pady=2)
        
        tk.Button(
            btn_frame, text="←", command=self._move_to_available, **btn_style
        ).pack(pady=2)
        
        tk.Button(
            btn_frame, text="⇒", command=self._move_all_to_selected, **btn_style
        ).pack(pady=2)
        
        tk.Button(
            btn_frame, text="⇐", command=self._move_all_to_available, **btn_style
        ).pack(pady=2)
        
        # Selected list (right)
        selected_frame = tk.Frame(container, bg=self.theme["bg_card"])
        selected_frame.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        
        tk.Label(
            selected_frame, text=selected_label,
            font=FONTS.get_font("body", bold=True),
            bg=self.theme["bg_card"], fg=selected_color,
        ).pack(anchor=tk.W)
        
        # Selected listbox with scrollbar
        sel_list_frame = tk.Frame(selected_frame, bg=self.theme["bg_card"])
        sel_list_frame.pack(fill=tk.BOTH, expand=True, pady=2)
        
        sel_scrollbar = ttk.Scrollbar(sel_list_frame, orient=tk.VERTICAL)
        sel_scrollbar.pack(side=tk.RIGHT, fill=tk.Y)
        
        self.selected_listbox = tk.Listbox(
            sel_list_frame, height=height,
            font=FONTS.get_font("small"),
            selectmode=tk.EXTENDED,
            bg=self.theme["bg_input"],
            fg=self.theme["text_primary"],
            selectbackground=selected_color,
            selectforeground="white",
            relief=tk.FLAT,
            highlightthickness=1,
            highlightbackground=self.theme["border"],
            yscrollcommand=sel_scrollbar.set,
            exportselection=False,
        )
        self.selected_listbox.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        sel_scrollbar.config(command=self.selected_listbox.yview)
    
    def _move_to_selected(self):
        """Move selected items from available to selected."""
        selected = list(self.available_listbox.curselection())
        items = [self.available_listbox.get(i) for i in selected]
        for item in items:
            self.selected_listbox.insert(tk.END, item)
        for i in reversed(selected):
            self.available_listbox.delete(i)
    
    def _move_to_available(self):
        """Move selected items from selected to available."""
        selected = list(self.selected_listbox.curselection())
        items = [self.selected_listbox.get(i) for i in selected]
        for item in items:
            self.available_listbox.insert(tk.END, item)
        for i in reversed(selected):
            self.selected_listbox.delete(i)
    
    def _move_all_to_selected(self):
        """Move all items to selected."""
        items = list(self.available_listbox.get(0, tk.END))
        for item in items:
            self.selected_listbox.insert(tk.END, item)
        self.available_listbox.delete(0, tk.END)
    
    def _move_all_to_available(self):
        """Move all items to available."""
        items = list(self.selected_listbox.get(0, tk.END))
        for item in items:
            self.available_listbox.insert(tk.END, item)
        self.selected_listbox.delete(0, tk.END)
    
    def set_available_items(self, items: List[str]):
        """Set the available items list."""
        self.available_listbox.delete(0, tk.END)
        for item in items:
            self.available_listbox.insert(tk.END, item)
    
    def set_selected_items(self, items: List[str]):
        """Set the selected items list."""
        self.selected_listbox.delete(0, tk.END)
        for item in items:
            self.selected_listbox.insert(tk.END, item)
    
    def get_available(self) -> List[str]:
        """Get list of available items."""
        return list(self.available_listbox.get(0, tk.END))
    
    def get_selected(self) -> List[str]:
        """Get list of selected items."""
        return list(self.selected_listbox.get(0, tk.END))
    
    def clear(self):
        """Clear both lists."""
        self.available_listbox.delete(0, tk.END)
        self.selected_listbox.delete(0, tk.END)



# -*- coding: utf-8 -*-
"""
Report Configuration Module.

Defines configuration classes for report generation.
"""

from __future__ import annotations

from dataclasses import dataclass, field
from enum import Enum
from pathlib import Path
from typing import Any, Dict, List, Optional, Union


class ReportLevel(Enum):
    """Report detail levels."""
    BASIC = "basic"
    STANDARD = "standard"
    EXTENDED = "extended"
    FULL = "full"
    CUSTOM = "custom"


class ReportFormat(Enum):
    """Supported report formats."""
    XLSX = "xlsx"
    DOCX = "docx"
    PPTX = "pptx"
    LATEX = "tex"
    HTML = "html"
    PDF = "pdf"  # Via LaTeX or HTML
    ALL = "all"


@dataclass
class ReportStyle:
    """Styling configuration for reports."""
    
    # Colors (hex without #)
    primary_color: str = "1F4E79"
    secondary_color: str = "2E75B6"
    accent_color: str = "70AD47"
    background_color: str = "FFFFFF"
    text_color: str = "333333"
    
    # Section colors
    section_colors: Dict[str, str] = field(default_factory=lambda: {
        "main_info": "E2EFDA",
        "counts": "FFF2CC",
        "stats": "FCE4D6",
        "plots": "DDEBF7",
        "networks": "E2D5F1",
        "advanced": "F4CCCC",
        "time": "D9EAD3",
        "laws": "CFE2F3",
    })
    
    # Typography
    title_font: str = "Arial"
    body_font: str = "Arial"
    title_size: int = 24
    heading1_size: int = 18
    heading2_size: int = 14
    body_size: int = 11
    
    # Spacing
    margin_top: float = 1.0  # inches
    margin_bottom: float = 1.0
    margin_left: float = 1.0
    margin_right: float = 1.0
    
    # Logo/branding
    logo_path: Optional[str] = None
    company_name: Optional[str] = None
    
    @classmethod
    def default(cls) -> "ReportStyle":
        """Default professional style."""
        return cls()
    
    @classmethod
    def academic(cls) -> "ReportStyle":
        """Academic/publication style."""
        return cls(
            primary_color="000000",
            secondary_color="666666",
            title_font="Times New Roman",
            body_font="Times New Roman",
            title_size=16,
            heading1_size=14,
            heading2_size=12,
            body_size=11,
        )
    
    @classmethod
    def modern(cls) -> "ReportStyle":
        """Modern/startup style."""
        return cls(
            primary_color="6366F1",  # Indigo
            secondary_color="8B5CF6",  # Purple
            accent_color="10B981",  # Emerald
            title_font="Helvetica",
            body_font="Helvetica",
        )


@dataclass
class ReportConfig:
    """Configuration for report generation."""
    
    # Basic settings
    title: str = "Bibliometric Analysis Report"
    subtitle: Optional[str] = None
    author: Optional[str] = None
    date: Optional[str] = None  # Auto-generated if None
    
    # Content control
    level: ReportLevel = ReportLevel.STANDARD
    include_toc: bool = True
    include_summary: bool = True
    include_methodology: bool = False
    include_appendix: bool = False
    
    # Item limits
    top_n_tables: int = 20  # Max rows in top-N tables
    top_n_plots: int = 15   # Max items in bar charts
    
    # Plot settings
    plot_dpi: int = 300
    plot_format: str = "png"  # png, svg, pdf
    regenerate_plots: bool = True  # Regenerate or use existing
    
    # Styling
    style: ReportStyle = field(default_factory=ReportStyle.default)
    
    # Output settings
    output_dir: Optional[str] = None
    filename_prefix: str = "report"
    
    # Template
    template_path: Optional[str] = None  # Custom template xlsx
    
    # Format-specific settings
    xlsx_settings: Dict[str, Any] = field(default_factory=lambda: {
        "include_charts": True,
        "autofit_columns": True,
        "freeze_panes": True,
        "conditional_formatting": True,
    })
    
    docx_settings: Dict[str, Any] = field(default_factory=lambda: {
        "page_size": "A4",  # A4, Letter
        "orientation": "portrait",
        "include_page_numbers": True,
        "include_header": True,
        "include_footer": True,
    })
    
    pptx_settings: Dict[str, Any] = field(default_factory=lambda: {
        "aspect_ratio": "16:9",  # 16:9, 4:3
        "include_notes": True,
        "include_slide_numbers": True,
        "max_slides": 50,
    })
    
    latex_settings: Dict[str, Any] = field(default_factory=lambda: {
        "document_class": "article",
        "paper_size": "a4paper",
        "font_size": "11pt",
        "bibliography_style": "apalike",
        "compile_pdf": False,
    })
    
    html_settings: Dict[str, Any] = field(default_factory=lambda: {
        "theme": "light",  # light, dark
        "interactive": True,
        "include_search": True,
        "single_file": True,  # Embed all resources
        "include_download_links": True,
    })
    
    def with_level(self, level: Union[str, ReportLevel]) -> "ReportConfig":
        """Return new config with different level."""
        if isinstance(level, str):
            level = ReportLevel(level)
        return ReportConfig(
            **{**self.__dict__, "level": level}
        )
    
    def with_style(self, style: ReportStyle) -> "ReportConfig":
        """Return new config with different style."""
        return ReportConfig(
            **{**self.__dict__, "style": style}
        )
    
    @classmethod
    def basic(cls, title: str = "Quick Analysis") -> "ReportConfig":
        """Minimal report configuration."""
        return cls(
            title=title,
            level=ReportLevel.BASIC,
            include_toc=False,
            include_summary=True,
            top_n_tables=10,
            top_n_plots=10,
        )
    
    @classmethod
    def standard(cls, title: str = "Bibliometric Analysis") -> "ReportConfig":
        """Standard report configuration."""
        return cls(
            title=title,
            level=ReportLevel.STANDARD,
        )
    
    @classmethod
    def comprehensive(cls, title: str = "Comprehensive Bibliometric Analysis") -> "ReportConfig":
        """Full detailed report configuration."""
        return cls(
            title=title,
            level=ReportLevel.FULL,
            include_methodology=True,
            include_appendix=True,
            top_n_tables=50,
            top_n_plots=25,
        )
    
    @classmethod
    def presentation(cls, title: str = "Research Overview") -> "ReportConfig":
        """Configuration optimized for presentations."""
        return cls(
            title=title,
            level=ReportLevel.BASIC,
            include_toc=False,
            top_n_tables=10,
            top_n_plots=10,
            pptx_settings={
                "aspect_ratio": "16:9",
                "include_notes": True,
                "include_slide_numbers": True,
                "max_slides": 20,
            }
        )



# -*- coding: utf-8 -*-
"""
Basic plotting functions - distributions, bar charts, scatter plots.

This module contains methods for:
- Distribution plots (box, violin)
- Top items bar charts
- Scatter plots
- Average citations plots
"""

from __future__ import annotations

import os
from typing import Any, Dict, List, Optional, Union

import pandas as pd


class BasicPlotsMixin:
    """Mixin class providing basic plotting methods."""
    
    def plot_average_citations_per_year(
        self,
        filename_base: str = "average citations per document",
        **kwargs: Any,
    ) -> None:
        """
        Plot average citations per document by publication year.

        Parameters
        ----------
        filename_base : str or None
            Base filename for saving. If None, plot is not saved.
        **kwargs :
            Additional arguments for plotbib.plot_average_citations_per_year.
        """
        from biblium import plotbib, utilsbib
        
        grouped = utilsbib.compute_average_citations_per_year(self.df)
        if filename_base is not None and self.res_folder is not None:
            filename_base = os.path.join(self.res_folder, "plots", filename_base)
            plotbib.plot_average_citations_per_year(grouped, **kwargs)

    def dist_plots(
        self,
        grouping_vars: List[str] = None,
        numeric_vars: List[str] = None,
        list_grouping_vars: Optional[List[str]] = None,
        max_groups: int = 5,
        order_by_size: bool = True,
        plot_type: str = "box",
        **kwargs: Any,
    ) -> None:
        """
        Generate box or violin plots for combinations of numeric and grouping variables.
    
        Parameters
        ----------
        grouping_vars : list of str
            Categorical variables for grouping (single value per row).
        numeric_vars : list of str
            Numerical variables to plot.
        list_grouping_vars : list of str, optional
            Variables with delimited lists.
        max_groups : int
            Maximum groups per plot.
        order_by_size : bool
            Order groups by size.
        plot_type : {"box", "violin"}
            Type of distribution plot.
        **kwargs :
            Additional plot arguments.
        """
        from biblium import plotbib
        
        if grouping_vars is None:
            grouping_vars = ["Source title", "Document Type"]
        if numeric_vars is None:
            numeric_vars = ["Year", "Cited by"]
        
        plot_func = {
            "box": plotbib.plot_boxplot,
            "violin": plotbib.plot_violinplot,
        }.get(plot_type)
    
        if plot_func is None:
            raise ValueError("plot_type must be 'box' or 'violin'")
    
        if list_grouping_vars is None:
            list_grouping_vars = []
    
        # Keep only existing columns
        grouping_vars = [c for c in grouping_vars if c in self.df.columns]
        numeric_vars = [c for c in numeric_vars if c in self.df.columns]
    
        def _explode_list_column(df, col, sep):
            """Explode a delimited string column into multiple rows."""
            tmp = df.dropna(subset=[col]).copy()
            tmp[col] = tmp[col].astype(str).str.split(sep)
            tmp = tmp.explode(col)
            tmp[col] = tmp[col].str.strip()
            tmp = tmp[tmp[col] != ""]
            return tmp
    
        # Plots for standard grouping vars
        for group_var in grouping_vars:
            for numeric_var in numeric_vars:
                if self.res_folder is not None:
                    filename_base = os.path.join(
                        self.res_folder, "plots",
                        f"{numeric_var} by {group_var}_{plot_type}",
                    )
                else:
                    filename_base = None
                plot_func(
                    self.df,
                    value_column=numeric_var,
                    group_by=group_var,
                    max_groups=max_groups,
                    order_by_size=order_by_size,
                    filename_base=filename_base,
                    dpi=getattr(self, "dpi", 600),
                    **kwargs,
                )
    
        # Plots for list-like grouping vars
        resolved_list_grouping = []
        for gv in list_grouping_vars:
            actual_col = None
            display_name = gv
    
            if gv in ("Author Keywords", "author keywords"):
                if "Processed Author Keywords" in self.df.columns:
                    actual_col = "Processed Author Keywords"
                elif "Author Keywords" in self.df.columns:
                    actual_col = "Author Keywords"
                display_name = "Author Keywords"
            elif gv in ("Index Keywords", "index keywords"):
                if "Processed Index Keywords" in self.df.columns:
                    actual_col = "Processed Index Keywords"
                elif "Index Keywords" in self.df.columns:
                    actual_col = "Index Keywords"
                display_name = "Index Keywords"
            else:
                if gv in self.df.columns:
                    actual_col = gv
    
            if actual_col is not None:
                resolved_list_grouping.append((display_name, actual_col))
    
        for display_name, actual_col in resolved_list_grouping:
            df_exploded = _explode_list_column(
                self.df, actual_col, getattr(self, "default_separator", "; ")
            )
            if display_name != actual_col:
                df_exploded = df_exploded.rename(columns={actual_col: display_name})
    
            for numeric_var in numeric_vars:
                if self.res_folder is not None:
                    filename_base = os.path.join(
                        self.res_folder, "plots",
                        f"{numeric_var} by {display_name}_{plot_type}",
                    )
                else:
                    filename_base = None
                plot_func(
                    df_exploded,
                    value_column=numeric_var,
                    group_by=display_name,
                    max_groups=max_groups,
                    order_by_size=order_by_size,
                    filename_base=filename_base,
                    dpi=getattr(self, "dpi", 600),
                    **kwargs,
                )

    def plot_top_items(
        self,
        items: str = "sources",
        x: str = "Number of documents",
        top_n: int = 10,
        ax=None,
        figsize: tuple = (10, 6),
        title: Optional[str] = None,
        filename: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        """
        Plot top N items as a horizontal bar chart.

        Parameters
        ----------
        items : str
            Type of items: "sources", "authors", "keywords", etc.
        x : str
            Column name for the metric.
        top_n : int
            Number of top items to show.
        ax : matplotlib Axes, optional
            Axes to plot on.
        figsize : tuple
            Figure size.
        title : str, optional
            Plot title.
        filename : str, optional
            Filename for saving.
        **kwargs :
            Additional plot arguments.
        """
        from biblium import plotbib
        import matplotlib.pyplot as plt
        
        # Get the appropriate counts DataFrame
        items_map = {
            "sources": ("sources_counts_df", "Source"),
            "authors": ("authors_counts_df", "Author(s) ID"),
            "author_keywords": ("author_keywords_counts_df", "Keyword"),
            "index_keywords": ("index_keywords_counts_df", "Keyword"),
            "keywords": ("keywords_counts_df", "Keyword"),
            "affiliations": ("affiliations_counts_df", "Affiliation"),
            "countries": ("ca_country_counts_df", "Country"),
        }
        
        if items not in items_map:
            raise ValueError(f"Unknown items type: {items}")
        
        attr_name, label_col = items_map[items]
        
        # Ensure counts exist
        if not hasattr(self, attr_name):
            count_method = f"count_{items}"
            if hasattr(self, count_method):
                getattr(self, count_method)()
        
        df = getattr(self, attr_name, None)
        if df is None or df.empty:
            return
        
        # Plot
        plot_df = df.head(top_n)
        
        if ax is None:
            fig, ax = plt.subplots(figsize=figsize)
        
        plotbib.plot_barh(
            plot_df, 
            x=x, 
            y=label_col,
            ax=ax,
            title=title or f"Top {top_n} {items.replace('_', ' ').title()}",
            **kwargs
        )
        
        if filename and self.res_folder:
            self._save_plot(filename)

    def scatter_plot_top_sources(
        self,
        x: str = "Number of documents",
        y: str = "Total citations",
        top_n: Optional[int] = None,
        **kwargs: Any,
    ) -> None:
        """Scatter plot for top sources."""
        self._scatter_plot_top_items("sources", x, y, top_n, **kwargs)
    
    def scatter_plot_top_authors(
        self,
        x: str = "Number of documents",
        y: str = "Total citations",
        top_n: Optional[int] = None,
        **kwargs: Any,
    ) -> None:
        """Scatter plot for top authors."""
        self._scatter_plot_top_items("authors", x, y, top_n, **kwargs)
    
    def scatter_plot_top_countries(
        self,
        x: str = "Number of documents",
        y: str = "Total citations",
        top_n: Optional[int] = None,
        **kwargs: Any,
    ) -> None:
        """Scatter plot for top countries."""
        self._scatter_plot_top_items("countries", x, y, top_n, **kwargs)
    
    def _scatter_plot_top_items(
        self,
        items: str,
        x: str,
        y: str,
        top_n: Optional[int],
        **kwargs: Any,
    ) -> None:
        """Internal scatter plot helper."""
        from biblium import plotbib
        
        items_map = {
            "sources": "sources_counts_df",
            "authors": "authors_counts_df",
            "countries": "ca_country_counts_df",
        }
        
        attr_name = items_map.get(items)
        if not attr_name:
            return
        
        df = getattr(self, attr_name, None)
        if df is None:
            return
        
        if top_n:
            df = df.head(top_n)
        
        plotbib.plot_scatter(df, x=x, y=y, **kwargs)



# -*- coding: utf-8 -*-
"""
GUI Panels Package
==================
Analysis and data panels for the workspace.
"""

from biblium.gui.panels.base import BasePanel

# Data panels
from biblium.gui.panels.data import LoadDataPanel, ViewDataPanel, FilterPanel, APIDataPanel

# Analysis panels
from biblium.gui.panels.analysis import CountsPanel, OverviewPanel, TrendsPanel, LawsPanel

# Network panels
from biblium.gui.panels.networks import NetworkPanel

# Mapping panels
from biblium.gui.panels.mapping import (
    ConceptualMapPanel, ThematicMapPanel, TopicsPanel, ClustersPanel
)

# Advanced panels
from biblium.gui.panels.advanced import (
    SleepingBeautyPanel, DisruptionPanel, ResearchFrontsPanel
)

# Report panels
from biblium.gui.panels.reports import ReportBuilderPanel, CustomReportPanel

__all__ = [
    "BasePanel",
    # Data
    "LoadDataPanel",
    "ViewDataPanel", 
    "FilterPanel",
    "APIDataPanel",
    # Analysis
    "CountsPanel",
    "OverviewPanel",
    "TrendsPanel",
    "LawsPanel",
    # Networks
    "NetworkPanel",
    # Mapping
    "ConceptualMapPanel",
    "ThematicMapPanel",
    "TopicsPanel",
    "ClustersPanel",
    # Advanced
    "SleepingBeautyPanel",
    "DisruptionPanel",
    "ResearchFrontsPanel",
    # Reports
    "ReportBuilderPanel",
    "CustomReportPanel",
]
